#!/usr/bin/perl
#
# Minimalist - MINImalist MAiling LIST manager.
# Copyright (c) 1999 Vladimir Litovka <doka@webest.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 675
# Mass Ave, Cambridge, MA 02139, USA.

use Fcntl ':flock';	# LOCK_* constants

$version = '2.0.2 (Banana)';
$config = "/usr/local/etc/minimalist.conf";

# Lists' status bits
$OPEN = 0;
$RO = 1;
$CLOSED = 2;
$MANDATORY = 4;

#####################################################
# Default values
#
$sendmail = '/usr/sbin/sendmail';
$domain = `hostname --fqdn`; chomp $domain;
$directory = '/var/spool/minimalist';
$admin = "root\@$domain";
$listpwd = $userpwd = '';
$security = 'careful';
$archive = 'no';
$archpgm = 'BUILTIN';
$status = $OPEN;
$copy_sender = 'yes';
$reply_to_list = 'no';
$errors_to = 'drop';
$modify_subject = 'yes';
$maxusers = 0;
$maxrcpts = 20;
$maxsize = 0;	# Maximum allowed size for message (incl. headers)
$auth_valid = 24;
$logfile = 'none';

#####################################################
# Various regular expressions
#
# for matching rounding spaces

$spaces = '^\s*(.+?)\s*$';

# for parsing two forms of mailing addresses:
#
# 1st form: Vladimir Litovka <doka@webest.com>
# 2nd form: doka@webest.com (Vladimir Litovka)

$first =  '^.*<([-=+\.\w\@]*)>.*$';
$second = '^([-=+\.\w\@]+)\s+\(.*\).*$';

# Help
$helpmsg = <<_EOF_ ;
This is the Minimalist Mailing List Manager.

All commands MUST APPEAR IN THE SUBJECT of mail messages.

Supported commands are:

subscribe <list> [<email>] :
    subscribe user to <list>

unsubscribe <list> [<email>] :
    unsubscribe user from <list>

auth <code> :
    confirm command, which need to be confirmed. This command isn't
    standalone, it must be used after certain other commands.

which [<email>] :
    get list of lists, to which user subscribed

info [<list>] :
    gives you information about existing lists or about <list>

who <list> :
    gives you the list of users subscribed to <list>

help :
    This message

Note, that <email> can be used only by trusted users. Otherwise, the
address in the 'From:' field will be used. The LIST command can be
used only by trusted users. Trusted users must supply the list password
in the first line of the message body, i.e:

*password: list_password

followed by any number of empty rows. The password will be removed from the
message, of course.
_EOF_

########################################################
# >>>>>>>>>>>>>>> SELF - CONFIGURING <<<<<<<<<<<<<<<<< #
########################################################

read_config($config, "global");

# Admin always in trusted users
push (@trusted, $admin);

$me = "minimalist\@$domain";

if ($errors_to eq 'drop') { $envelope_sender = "-f $me"; }
elsif ($errors_to eq 'admin') { $envelope_sender = "-f $admin"; }
else { $envelope_sender = ""; }

########################################################
# >>>>>>>>>>>>>>>> CHECK CONFIGURATION <<<<<<<<<<<<<<< #
########################################################

if ($ARGV[0] eq '-') {
  print "\nMinimalist v$version, pleased to meet you.\n\n".
	"================= Global configuration ================\n".
	"Directory: $directory\n".
	"Admin: $admin\n";
  die "$directory doesn't exist!\n" if (! -d $directory);

  while ( $ARGV[0] ) {
    @trusted = (); @blacklist = ();
    read_config($config, "global"); push (@trusted, $admin);

    $list = $ARGV[0];
    if ($list ne '-') {
      if (!chdir("$directory/$list")) {
        print "\nThere isn't such list \U$list\E\n";
        shift; next;
       }
      read_config("config");
      print "================= \U$list\E ================\n";
     }
    print "Sendmail: $sendmail\n".
	"Domain: $domain\n".
	"Password: ".($listpwd ? "yes, list protected" : "none")."\n".
	"Security: $security\n".
	"Logging: $logfile\n".
	"Archiving: $archive\n".
	  ($archive ne 'no' ? " * Archiver: $archpgm\n" : "").
	"Status:";
    if ($status) {
      print " read-only" if ($status & $RO);
      print " closed" if ($status & $CLOSED);
      print " mandatory" if ($status & $MANDATORY);
     }
    else { print " open"; }
    print "\nCopy to sender: $copy_sender\n".
	"Reply-To list: $reply_to_list\n".
	"Return errors to: $errors_to\n".
	"Modify subject: $modify_subject\n".
	"Maximal users per list: ".($maxusers ? $maxusers : "unlimited")."\n".
	"Maximal recipients per message: ".($maxrcpts ? $maxrcpts : "unlimited")."\n".
	"Maximal size of message: ".($maxsize ? "$maxsize bytes" : "unlimited")."\n".
	"Authentication request valid at least $auth_valid hours\n".
	"Trusted users:\n";
    foreach (@trusted) { print "\t . ".$_."\n"; }
    if ( @blacklist ) {
      print "Blacklist:\n";
      foreach (@blacklist) { print "\t . ".$_."\n"; }
     }
    print "\n";
    
    # Various checks
    $msg .= " * $sendmail doesn't exist!\n" if (! -x $sendmail);
    $msg .= " * Invalid domain '$domain'\n" if ($domain !~ /^(\w[-\w]*\.)+[a-z]{2,4}$/i);
    $msg .= " * Invalid security level '$security'\n" if ($security !~ /^none$|^careful$|^paranoid$/i);
    $msg .= " * Invalid 'copy to sender' value '$copy_sender'\n" if ($copy_sender !~ /^yes$|^no$/i);
    $msg .= " * Invalid 'modify subject' value '$modify_subject'\n" if ($modify_subject !~ /^yes$|^no$|^more$/i);
    $msg .= " * Invalid 'reply-to list' value '$reply_to_list'\n" if ($reply_to_list !~ /^yes$|^no$|\@/i);
    $msg .= " * Invalid 'errors to' value '$errors_to'\n" if ($errors_to !~ /^drop$|^admin$|^sender$/i);
    $msg .= " * Invalid authentication request validity time: $auth_valid\n" if ($auth_valid !~ /^[0-9]+$/);
    $msg .= " * Invalid archiving strategy '$archive'\n" if ($archive !~ /^no$|^daily$|^monthly$|^yearly$|^pipe$/i);
    if ($archive eq 'pipe') {
      ($arpg, ) = split(/\s+/, $archpgm, 2);
      $msg .= " * $arpg doesn't exists!\n" if (! -x $arpg);
     }

    die "Errors are:\n".$msg."\n" if ($msg);
    shift;
   }

  exit 0;
 }

########################################################
# >>>>>>>>>>>>>>>>>> START HERE <<<<<<<<<<<<<<<<<<<<<< #
########################################################

$list = $ARGV[0];
$auth_seconds = $auth_valid * 3600;	# Convert hours to seconds

$message .= $_ while (<STDIN>);
($header, $body) = split(/\n\n/, $message, 2); $header .= "\n";

undef $message;		# Clear memory(???), it doesn't used anymore

#####-------------------------------------#####
# Note: mail FROM, but in most cases REPLY-TO #
#####-------------------------------------#####

$from = $reply = $sender = $xsender = $subject = '';

# Extract From:
if ($header =~ /(^|\n)from: (.*\n([ \t]+.*\n)*)/i) {
  $from = $2; $from =~ s/$spaces/$1/ogs;
  $from =~ s/\n/ /g; $from =~ s/\s{2,}/ /g;
  $fullfrom = $from; }

# Extract Reply-To:
if ($header =~ /(^|\n)reply-to: (.*\n([ \t]+.*\n)*)/i) {
  $reply = $2; chomp $reply; }

# Extract Sender:
if ($header =~ /(^|\n)sender: (.*)\n/i) {
  $sender = $2; }

# Extract X-Sender:
if ($header =~ /(^|\n)x-sender: (.*)\n/i) {
  $xsender = $2; }
 
($reply eq '') ? ($mailto = $from) : ($mailto = $reply);

# Preparing From:
if ($from =~ /<.*>/) { $from =~ s/$first/$1/og; }
elsif ($from =~ /\(.*\)/) { $from =~ s/$second/$1/og; }
$from =~ s/$spaces/$1/gs;
$from = lc($from);

exit 0 if (($xsender eq $me) || ($from eq $me));	# LOOP detected
exit 0 if (($from =~ /MAILER-DAEMON/i) ||
	   ($sender =~ /MAILER-DAEMON/i) ||
	   ($xsender =~ /MAILER-DAEMON/i));		# ignore messages from MAILER-DAEMON

foreach (@blacklist) {					# ignore messages from Black List
  exit 0 if (($from =~ /$_$/i) ||
  	     ($sender =~ /$_$/i) ||
  	     ($xsender =~ /$_$/i));
 }

# Get (multiline) subject
if ($header =~ /(^|\n)subject: (.*\n([ \t]+.*\n)*)/i) {
  $subject = $2; $subject =~ s/$spaces/$1/gs; }

# Get password if it's there and strip this line from message's body
if ($body =~ s/(^\s*\*password: )(.*)\n+(.*)/$3/i) {
  $userpwd = $2;
 }

$body =~ s/\n*$//g;

#########################################################################
########################## Message to list ##############################
#
if ($list) {

 if (! -d "$directory/$list" ) {
   $msg = <<_EOF_ ;
From: Minimalist Manager <$me>
To: $admin
Subject: Possible error in system settings

ERROR:
    Minimalist was called with '$list' argument, but there isn't such
    list in '$directory'.

SOLUTION:
    Check your 'aliases' file - there is possible typo.

_EOF_
   goto SendMessage;	# Line 611, send message.
  }

 chdir("$directory/$list");
 read_config("config");

 if ($modify_subject ne 'no') {
   if ($modify_subject eq 'more') {
     $subject =~ s/^.*:\s+(\[$list\])/$1/ig }	# Remove leading "Re: "

   else {	# change anything before [...] to Re:
     $subject =~ s/^(.*:\s+)+(\[$list\])/Re: $2/ig; }

   if ($subject !~ /^(.*:\s+)?\[$list\] /i) {
     $subject = "[$list] ".$subject; }		# Modify subject if it isn't
  }
  
 open LIST, "list";
 $users .= $_ while (<LIST>);
 close LIST;
 @members = split ("\n", $users);
 
 if (($security ne 'none') && !grep(/^$from$/i, @members)) {
   $msg = <<_EOF_ ;
From: Minimalist Manager <$me>
To: $mailto
Subject: $subject

ERROR:
    You ($from) are not subscribed to this list.

SOLUTION:
    Send a message to minimalist\@$domain with a subject
    of 'info' (no quotes) for information about subscribing.

===========================================================================

$body
_EOF_
  } 
 elsif (($status & $RO) && (!grep(/^$from$/i, @trusted) || ($userpwd ne $listpwd))) {
   $msg = <<_EOF_ ;
From: Minimalist Manager <$me>
To: $mailto
Subject: $subject

ERROR:
    You are not allowed to write to this list.

===========================================================================

$body
_EOF_
  }
 elsif ($maxsize && (length($header) + length($body) > $maxsize)) {
   $header = substr($header, 0, 4000);	# Send first 4000 bytes of header, not more
   $msg = <<_EOF_ ;
From: Minimalist Manager <$me>
To: $mailto
Subject: $subject

ERROR:
    Message size is larger than maximum allowed ($maxsize bytes).

SOLUTION:
    Either send a smaller message or split your message into multiple
    smaller ones.

===========================================================================
Header of your message follows:

$header
_EOF_
  }
 else {		# Ok, all checks done.

   if ($archive eq 'pipe') { arch_pipe(); }
   elsif ($archive ne 'no') { archive(); }

   # Send message to members except those who already
   # received this message directly from sender

   # Get all recipients from To: and Cc: (expect multi-line)
   $header =~ s/(^|\n)to: (.*\n([ \t]+.*\n)*)/$1/i; $rc = $2;
     @recip = split(/[\n,]/, $rc);
   $header =~ s/(^|\n)cc: (.*\n([ \t]+.*\n)*)/$1/i; $rc = $2;
     push (@recip, split(/[\n,]/, $rc));

   foreach (@recip) {
     if ( /<.*>/ ) { s/$first/$1/g; }
     elsif ( /\(.*\)/ ) { s/$second/$1/g; }
     s/$spaces/$1/gs;
     push (@rcpts, lc($_));
    }

   if ($copy_sender eq 'no') { push (@rcpts, $from) }	# @rcpts will be _excluded_

   @members = sort @members; @rcpts = sort @rcpts;

   for ($r=0, $m=0; $m < @members; ) {
     if ($members[$m] lt $rcpts[$r] || $r >= @rcpts) {
       push (@recipients, $members[$m]);
       $m++; }
     elsif ($members[$m] eq $rcpts[$r]) { $r++; $m++; }
     elsif ($members[$m] gt $rcpts[$r]) { $r++ };
    }

   # Remove conflicting X- headers
   $header =~ s/x-mailing-list.*\n([ \t]+.*\n)*//ig;
   $header =~ s/x-sender: .*\n([ \t]+.*\n)*//ig;

   $header =~ s/\n*$//g;
   $header .= "\nTo: $list\@$domain\n";		# Mail to himself, loop is handling

   if ($reply_to_list ne 'no') {
     $header =~ s/(^|\n)reply-to: .*\n([ \t]+.*\n)*/$1/ig;
     if ($reply_to_list eq 'yes') { $header .= "Reply-To: $list\@$domain\n"; }
     else { $header .= "Reply-To: $reply_to_list\n"; }
    }

   if ($modify_subject ne 'no') {
     $header =~ s/(^|\n)subject: .*\n([ \t]+.*\n)*/$1/ig;
     $header .= "Subject: $subject\n";
    }

   if ( !$sender ) {	# If there isn't Sender: in original message
     $header .= "Sender: $from\n"; }

   #########################################################
   # Send message to recipients ($maxrcpts per message)

   $footer = read_info("footer");
   $maxrcpts = $#recipients + 1 if ($maxrcpts == 0);	# Unlimited?

   foreach $one (@recipients) {
     if ($rcs == $maxrcpts) {
       sendPortion();
       $bcc = ''; $rcs = 0;	# Clear counters
      }
     if ($one ne '') {
       $bcc .= "$one,"; $rcs++; }
    }

   sendPortion() if ($bcc ne '');	# Send to rest subscribers

   $msg = '';	# Clear message, don't send anything anymore
  }

} else {

#########################################################################
######################## Message to Minimalist ##########################
#
# Allowed commands:
#	subscribe <list> [<e-mail>]
#	unsubscribe <list> [<e-mail>]
#	auth <code>
#	which [<e-mail>]
#	info [<list>]
#	who <list>
#	help

 $subject =~ s/^.*: //g;	# Strip leading 'Anything: '

 $list = ''; $email = '';
 ($cmd, $list, $email) = split (/\s+/, $subject, 3);
 $cmd = lc($cmd); $list = lc($list);

 if ($email ne '') {
   if ($email =~ /<.*>/) { $email =~ s/$first/$1/og; }
   elsif ($email =~ /\(.*\)/) { $email =~ s/$second/$1/og; }
   $email =~ s/$spaces/$1/gs; $email = lc($email);
  }

 $msg = "From: Minimalist Manager <$me>\n".
	"To: $mailto\n".
	"Subject: Re: $subject\n".
	"X-Sender: $me\n".
	"X-Mailing-List-Server: Minimalist v$version\n";
 
 if ($cmd eq 'help') {
   $msg .= "\n".$helpmsg; }

 elsif ($cmd eq 'auth' && ($authcode = $list)) {
   ($cmd, $list, $from) = getAuth($authcode);
   if ($cmd eq 'subscribe' || $cmd eq 'unsubscribe') { 
     chdir "$directory/$list";
     read_config("config");
     $owner = "$list-owner\@$domain";
     &logCommand("$cmd $list $from") if (eval("$cmd(0)") && $logfile ne 'none');
    }
   else {
     $msg .= "\nERROR:\n\tThere is no authentication request with such code: $authcode\n".
             "\nSOLUTION:\n\tNone.";
    }
  }

 elsif ($cmd eq 'which') {	# $list means $email here
   if ($list && $list ne $from && (!grep(/^$from$/i, @trusted) || ($userpwd ne $listpwd))) {
     $msg .= "\nERROR:\n\tYou are not allowed to get subscription of other users.\n".
             "\nSOLUTION:\n\tNone.";
    }
   else {
     $from = $list if ($list);
     &logCommand($subject) if ($logfile ne 'none');
     $msg .= "\nCurrent subscription of user $from :\n\n";
     chdir $directory;
     opendir DIR, ".";
     while ($dir = readdir DIR) {
       if (-d $dir && $dir !~ /^\./) {
         open LIST, "$dir/list";
         while (<LIST>) {
           chomp($_);
           if ($from eq $_) { $msg .= "* \U$dir\E\n"; last; }
          }
         close LIST;
        }
      }
     closedir DIR;
    }
  }

 elsif ( ($list ne '') && (! -d "$directory/$list") ) {
   $msg .= <<_EOF_ ;

ERROR:
    There is no such list \U$list\E here.

SOLUTION:
    Send a message to minimalist\@$domain with a subject
    of 'info' (no quotes) for a list of available mailing lists.
_EOF_
  }

 elsif ( (($cmd eq 'subscribe') || ($cmd eq 'unsubscribe')) && ($list ne '') ) {
   $melist = "$list\@$domain";

   # Check for possible loop
   exit 0 if (($from eq $melist) || ($email eq $me) || ($email eq $melist));

   chdir "$directory/$list";
   read_config("config");
   $owner = "$list-owner\@$domain";

   if (grep(/^$from$/i, @trusted) && ($userpwd eq $listpwd)) {
     &logCommand($subject) if (eval("$cmd(1)") && $logfile ne 'none');
    }
   elsif (($email ne '') && ($email ne $from)) {
     $msg .=  "\nERROR:\n\tYou aren't allowed to subscribe other persons.\n".
	      "\nSOLUTION:\n\tNone.";
    }
   elsif (($cmd eq 'subscribe') && ($status & $CLOSED)) {
     $msg .=  "\nERROR:\n\tSorry, this list is closed for you.\n".
	      "\nSOLUTION:\n\tAre you unsure? Please, complain to $owner";
    }
   elsif (($cmd eq 'unsubscribe') && ($status & $MANDATORY)) {
     $msg .=  "\nERROR:\n\tSorry, this list is mandatory for you.\n".
	      "\nSOLUTION:\n\tAre you unsure? Please, complain to $owner";
    }
   else {
     if ($security ne 'paranoid') {
       &logCommand($subject) if (eval("$cmd(0)") && $logfile ne 'none')
      }
     else {
       $authcode = genAuth();
       $msg = <<_EOF_ ;
From: Minimalist Manager <$me>
To: $from
Subject: auth $authcode

Your request

	$subject

must be authenticated. To accomplish this, send another request
to $me with the following subject:

       auth $authcode

This authentication request valid for next $auth_valid hours from
now. After this term it will be discarded.
_EOF_
      }        # paranoid security
    }
  }	# subscribe/unsubscribe

 elsif ($cmd eq 'info') {
   &logCommand($subject) if ($logfile ne 'none');
   if ($list ne '') {
     $msg .= "\nHere is the available information about \U$list\E\n\n";
     $msg .= read_info("$directory/$list/info");
    }
   else {
     $msg .= "\nThese are the mailing lists available at $domain:\n\n";
     if (open(INFO, "$directory/lists.lst")) {
       while (<INFO>) {
         $msg .= $_ if (! /^#/); }
       close INFO;
      }
    }
  }
 elsif (($cmd eq 'who' || $cmd eq 'list') && ($list ne '')) {
   chdir "$directory/$list";
   read_config("config");

   if (grep(/^$from$/i, @trusted) && ($userpwd eq $listpwd)) {
     &logCommand($subject) if ($logfile ne 'none');
     $msg .= "\nUsers, subscribed to \U$list\E:\n\n";
     if (open(LIST, "list")) {
       $msg .= $_ while (<LIST>);
       close LIST;
      }
    }
   else {
     $msg .= "\nERROR:\n\tYou aren't allowed to get listing of subscribed users.\n".
     	     "\nSOLUTION:\n\tNone.";
    }
  }
 else {
   $msg .= "\nERROR:\n\tBad syntax or unknown instruction.\n".
   	   "\nSOLUTION:\n\n$helpmsg";
  }

 cleanAuth();		# Clean old authentication requests
}

SendMessage:

if ($msg ne '') {
  $msg =~ s/\n*$//g;
  open MAIL, "| $sendmail -t $envelope_sender";
  print MAIL "$msg\n\n-- \nSincerely, the Minimalist\n";
  close MAIL;
 }

exit 0;

#########################################################################
###################### Supplementary functions ##########################

#................... SUBSCRIBE .....................
sub subscribe {

 my ($trusteduser) = @_;
 my ($cc);

 if ($email ne '') {
   $cc = "$email,";
   $from = $email;
  }

 if (open LIST, "list") {
   $users .= $_ while (<LIST>);
   close LIST;
   @members = split ("\n", $users);
   if (grep(/^$from$/i, @members)) {
     $deny = 1;
     $cause = "you already subscribed to \U$list\E";
    }
   elsif ((! $trusteduser) && ($maxusers > 0) && (@members >= $maxusers)) {
     $deny = 1; $cc .= "$owner,";
     $cause = "there are already maximal count of subscribers ($maxusers) at \U$list\E";
    }
   else { open LIST, ">>list"; }
  }
 else { open LIST, ">list"; } 

 if ($cc) {
   chop $cc;
   $msg .= "Cc: $cc\n";
  }

 $msg .= "\nDear $from,\n\n";

 if (! $deny) {
   &lockf(LIST, 'lock'); print LIST "$from\n"; &lockf(LIST);
   $msg .= <<_EOF_ ;
    You have subscribed to \U$list\E successfully.

Please, note the following:
_EOF_
   $msg .= "\n".read_info("info");
  }
 else {
   $msg .= <<_EOF_ ;
You haven't subscribed to \U$list\E due to the following reason:

    * $cause.

If you have comments or questions, please, send them to the list
administrator $owner.
_EOF_
  }

 !$deny;
}

#................... UNSUBSCRIBE .....................
sub unsubscribe {
 if ($email ne '') {
   $msg .= "Cc: $email\n" ;
   $from = $email;
  }

 if (open LIST, "list") {
   $users .= $_ while (<LIST>);
   close LIST;

   $users =~ s/$from\n//g;
   open LIST, ">list";
   &lockf(LIST, 'lock'); print LIST $users; &lockf(LIST);
  }

 $msg .= "\nUser $from has successfully unsubscribed.\n\nGood bye!";

 1;
}

#................... READ CONFIG .....................
sub read_config {

my ($fname, $global) = @_;

if (open(CONF, $fname)) {
  while (<CONF>) {
    if($_ =~ /^#/) {
    } elsif (($_ =~ /^directory/i) && $global) {
     ($directive, $directory) = split(/=/, $_);
     $directory =~ s/$spaces/$1/gs;

    } elsif (($_ =~ /^logfile/i) && $global) {
     ($directive, $logfile) = split(/=/, $_);
     $logfile =~ s/$spaces/$1/gs;

    } elsif (($_ =~ /^admin/i) && $global) {
     ($directive, $admin) = split(/=/, $_);
     $admin =~ s/$spaces/$1/gs;

    } elsif ($_ =~ /^sendmail/i) {
     ($directive, $sendmail) = split(/=/, $_);
     $sendmail =~ s/$spaces/$1/gs;

    } elsif ($_ =~ /^domain/i) {
     ($directive, $domain) = split(/=/, $_);
     $domain =~ s/$spaces/$1/gs;

    } elsif ($_ =~ /^password/i) {
     ($directive, $listpwd) = split(/=/, $_);
     $listpwd =~ s/$spaces/$1/gs;

    } elsif ($_ =~ /^security/i) {
     ($directive, $security) = split(/=/, $_);
     $security =~ s/$spaces/$1/gs; $security = lc($security);

    } elsif ($_ =~ /^errors to/i) {
     ($directive, $errors_to) = split(/=/, $_);
     $errors_to =~ s/$spaces/$1/gs; $errors_to = lc($errors_to);

    } elsif ($_ =~ /^archive/i) {
     ($directive, $archive) = split(/=/, $_);
     $archive =~ s/$spaces/$1/gs;

     if ($archive =~ s/^pipe\s+//i) {
       $archpgm = $archive;
       $archive = "pipe"; }
     else {
       $archpgm = 'BUILTIN';
       $archive =~ s/$spaces/$1/gs; $archive = lc($archive);
      }

    } elsif ($_ =~ /^maxusers/i) {
     ($directive, $maxusers) = split(/=/, $_);
     $maxusers =~ s/$spaces/$1/gs;

    } elsif ($_ =~ /^maxrcpts/i) {
     ($directive, $maxrcpts) = split(/=/, $_);
     $maxrcpts =~ s/$spaces/$1/gs;

    } elsif ($_ =~ /^maxsize/i) {
     ($directive, $maxsize) = split(/=/, $_);
     $maxsize =~ s/$spaces/$1/gs;

    } elsif ($_ =~ /^status/i) {
     ($directive, $status) = split(/=/, $_);
     $status =~ s/\s+//g;	# Remove any spaces
     $status = lc($status);

     # Calculate mask for status
     %strel = ("open", $OPEN, "ro", $RO, "closed", $CLOSED, "mandatory", $MANDATORY);
     @starr = split(/,/, $status);
     $status = 0;
     foreach (@starr) { $status += $strel{$_}; }

    } elsif ($_ =~ /^copy to sender/i) {
     ($directive, $copy_sender) = split(/=/, $_);
     $copy_sender =~ s/$spaces/$1/gs; $copy_sender = lc($copy_sender);

    } elsif ($_ =~ /^reply-to list/i) {
     ($directive, $reply_to_list) = split(/=/, $_);
     $reply_to_list =~ s/$spaces/$1/gs; $reply_to_list = lc($reply_to_list);

     # In global config only 'yes' or 'no' allowed
     if ($global && ($reply_to_list ne 'yes')) { $reply_to_list = 'no'; }

    } elsif ($_ =~ /^modify subject/i) {
     ($directive, $modify_subject) = split(/=/, $_);
     $modify_subject =~ s/$spaces/$1/gs; $modify_subject = lc($modify_subject);

    } elsif ($_ =~ /^auth valid/i && $global) {
     ($directive, $auth_valid) = split(/=/, $_);
     $auth_valid =~ s/$spaces/$1/gs; $auth_valid = lc($auth_valid);

    } elsif ($_ =~ /^trusted/i) {
     ($directive, $trust) = split(/=/, $_, 2);
     $trust =~ s/\s+//g;
     push (@trusted, expand_lists(split(':', $trust)));

    } elsif (($_ =~ /^blacklist/i) && $global) {
     ($directive, $black) = split(/=/, $_, 2);
     $black =~ s/\s+//g;
     push (@blacklist, expand_lists(split(':', $black)));
    }
  }
  close CONF;
 }
}

#..........................................................
sub expand_lists {
 my (@junk) = @_;
 my (@result);

 foreach $s (@junk) {
   if ( $s =~ s/^\@// ) {	# Expand items, starting with '@'
     if (open(IN, $s)) {
       while (<IN>) {
         chomp $_; $result[@result] = $_; }
       close IN;
      }
    }
   elsif ($s ne '') { $result[@result] = $s; }
  }
 @result;
}

#.......... Read file and substitute all macroses .........
sub read_info {
 my ($fname) = @_;
 my ($tail);

 if (open(TAIL, $fname)) {
   $tail .= $_ while (<TAIL>);
   close TAIL;

   $tail =~ s/\\a/$admin/ig;
   $tail =~ s/\\d/$domain/ig;
   $tail =~ s/\\l/$list/ig;
   $tail =~ s/\\o/$list-owner\@$domain/ig;
  }
 $tail;
}

#.......... Send ready portion of message ............
sub sendPortion {

 chop $bcc;
 $temphdr = $header . "Bcc: $bcc\n";
 $temphdr .= "X-Sender: $me\n".
 "X-Mailing-List: $list\@$domain\n".
 "X-Mailing-List-Owner: $list-owner\@$domain\n".
 "X-Mailing-List-Server: Minimalist v$version\n\n";
 $msg = $temphdr.$body."\n\n".$footer;

 open MAIL, "| $sendmail -t $envelope_sender";
 print MAIL $msg."\n";
 close MAIL;
}

#.................... Built-in archiver ..........................
sub archive {

 @date = localtime;
 $year = 1900 + $date[5];
 $month = 1 + $date[4];
 $day = $date[3];

 $path = "archive/";
 mkdir($path, 0755) if (! -d $path);

 @types = ("yearly", "monthly", "daily");
 %rel = ($types[0], $year, $types[1], $month, $types[2], $day);

 foreach $key (@types) {
   $path .= $rel{$key}."./";
   mkdir($path, 0755) if (! -d $path);
   last if ($key eq $archive);
  }

 if (open(NUM, $path."SEQUENCE")) {
   read NUM, $msgnum, 16;
   $msgnum = int($msgnum);
   close NUM;
  }
 else { $msgnum = 0 }

 open ARCHIVE, ">$path".$msgnum;
 print ARCHIVE $header."\n".$body;
 close ARCHIVE;

 open NUM, ">$path"."SEQUENCE";
 print NUM $msgnum+1;
 close NUM;
}

#.................... External archiver ..........................
sub arch_pipe {

 open (ARCHIVE, "| $archpgm");
 print ARCHIVE $header."\n".$body;
 close (ARCHIVE);
}

#.................... Generate authentication code ...............
sub genAuth {

 my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime;
 my ($authcode) = $mon.$mday.$hour.$min.$sec."-$$";

 mkdir ("$directory/.auth", 0750) if (! -d "$directory/auth");

 open AUTH, ">$directory/.auth/$authcode";
 print AUTH "$cmd $list $from\n";
 close AUTH;

 $authcode;
}

#................. Check for authentication code ...............
sub getAuth {

 my ($cmd, $list, $from);
 my ($authcode) = @_;
 my ($authfile) = "$directory/.auth/$authcode";

 open AUTH, $authfile and do {
   $authtask = <AUTH>; chomp $authtask;
   close AUTH; unlink $authfile;

   ($cmd, $list, $from) = split(/ /, $authtask);
  };
}

#............... Clean old authentication requests .............
sub cleanAuth {

 my $now = time;
 my $dir = "$directory/.auth";
 my $mark = "$dir/.lastclean";

 if (! -f $mark) { open LC, "> $mark"; close LC; return; }
 else {
   my @ftime = stat(_);
   return if ($now - $ftime[9] < $auth_seconds);	# Return if too early
  }

 utime $now, $now, $mark;	# Touch .lastclean
 opendir DIR, $dir;
 while ($entry = readdir DIR) {
   if ($entry !~ /^\./ && -f "$dir/$entry") {
     @ftime = stat(_);
     unlink "$dir/$entry" if ($now - $ftime[9] > $auth_seconds);
    }
  }
 closedir DIR;
}

#............................ Locking .........................
sub lockf {
 my ($FD, $lock) = @_;

 if ($lock) {		# Lock FD
   flock $FD, LOCK_EX;
   seek $FD, 0, 2;
  }
 else {			# Unlock FD and close it
   flock $FD, LOCK_UN;
   close $FD;
  }
}

#......................... Logging activity ....................
sub logCommand {

 my ($command) = @_;

 $command =~ s/(\n|\|)/ /g;	# Prepare for logging
 $command =~ s/\s{2,}/ /g;

 open FILE, ">>$logfile"; &lockf(FILE, 'lock');
 $from = $email if ($email);
 print FILE localtime()."|$from|$command|$fullfrom\n";
 &lockf(FILE);
}
